"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonTemplateEngine = void 0;
const constants_1 = require("./constants");
const lexer_1 = require("./lexer");
const parser_1 = require("./parser");
const translator_1 = require("./translator");
const utils_1 = require("./utils");
class JsonTemplateEngine {
    constructor(fn) {
        this.fn = fn;
    }
    static create(templateOrExpr, options) {
        return new JsonTemplateEngine(this.compileAsAsync(templateOrExpr, options));
    }
    static createAsSync(templateOrExpr, options) {
        return new JsonTemplateEngine(this.compileAsSync(templateOrExpr, options));
    }
    static compileAsSync(templateOrExpr, options) {
        return Function(constants_1.DATA_PARAM_KEY, constants_1.BINDINGS_PARAM_KEY, this.translate(templateOrExpr, options));
    }
    static compileAsAsync(templateOrExpr, options) {
        return utils_1.CommonUtils.CreateAsyncFunction(constants_1.DATA_PARAM_KEY, constants_1.BINDINGS_PARAM_KEY, this.translate(templateOrExpr, options));
    }
    static parse(template, options) {
        const lexer = new lexer_1.JsonTemplateLexer(template);
        const parser = new parser_1.JsonTemplateParser(lexer, options);
        return parser.parse();
    }
    static translate(templateOrExpr, options) {
        if (typeof templateOrExpr === 'string') {
            return this.translateTemplate(templateOrExpr, options);
        }
        return this.translateExpression(templateOrExpr);
    }
    static translateTemplate(template, options) {
        return this.translateExpression(this.parse(template, options));
    }
    static translateExpression(expr) {
        const translator = new translator_1.JsonTemplateTranslator(expr);
        return translator.translate();
    }
    evaluate(data, bindings = {}) {
        return this.fn(data || {}, bindings);
    }
}
exports.JsonTemplateEngine = JsonTemplateEngine;
